Package: R2admb
Title: 'ADMB' to R Interface Functions
Version: 0.7.16.3
Authors@R: c(person("Ben Bolker",role=c("aut","cre"),
         email="bolker@mcmaster.ca"),
	 person("Hans","Skaug",role="aut"),
	 person("Jeff","Laake",role="aut"))
Description: A series of functions to call 'AD Model Builder' (i.e.,
    compile and run models) from within R, read the results back
    into R as 'admb' objects, and provide standard accessors (i.e.
    coef(), vcov(), etc.)
Depends: R (>= 3.0.1)
Imports: coda, lattice
Suggests: bbmle, lme4, ggplot2 (>= 3.4.0), testthat, knitr
SystemRequirements: AD Model Builder <http://admb-project.org>
VignetteBuilder: knitr
License: GPL
LazyLoad: yes
Repository: CRAN
NeedsCompilation: no
RoxygenNote: 6.0.1
Packaged: 2022-11-07 15:53:16 UTC; bolker
Author: Ben Bolker [aut, cre],
  Hans Skaug [aut],
  Jeff Laake [aut]
Maintainer: Ben Bolker <bolker@mcmaster.ca>
Date/Publication: 2022-11-07 16:40:06 UTC
Built: R 4.6.0; ; 2026-04-25 16:06:45 UTC; unix
