# Generated by roxygen2: do not edit by hand

S3method(AIC,admb)
S3method(coef,admb)
S3method(confint,admb)
S3method(deviance,admb)
S3method(logLik,admb)
S3method(plot,admb_hist)
S3method(print,admb)
S3method(print,summary.admb)
S3method(stdEr,admb)
S3method(summary,admb)
S3method(vcov,admb)
export(admb_version)
export(clean_admb)
export(compile_admb)
export(dat_write)
export(do_admb)
export(extract_gradient)
export(find_large_cor)
export(mcmc.control)
export(read_admb)
export(read_pars)
export(read_plt)
export(read_psv)
export(read_rep)
export(run.control)
export(run_admb)
export(setup_admb)
export(stdEr)
export(write_dat)
export(write_pin)
importFrom(coda,HPDinterval)
importFrom(coda,as.mcmc)
importFrom(lattice,xyplot)
importFrom(stats,AIC)
importFrom(stats,coef)
importFrom(stats,deviance)
importFrom(stats,logLik)
importFrom(stats,na.omit)
importFrom(stats,pnorm)
importFrom(stats,printCoefmat)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,vcov)
importFrom(utils,count.fields)
importFrom(utils,head)
importFrom(utils,read.table)
importFrom(utils,write.table)
